/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * Rotation3D.cpp
 *
 *  Created on: Apr 11, 2011
 *      Author: lziegler
 */

#include "Rotation3D.h"

using namespace airobots;
using namespace xmltio;
using namespace std;

Rotation3D::Rotation3D() {
	mat.identity();
}

Rotation3D::Rotation3D(double x, double y, double z, double angle,
		AngleUnit angleUnit) {
	setAxisAngle(x, y, z, angle, angleUnit);
}

Rotation3D::Rotation3D(const Vector3d &axis, double angle, AngleUnit angleUnit) {
	setAxisAngle(axis, angle, angleUnit);
}

Rotation3D::Rotation3D(const Quatd &quaternion) {
	setQuaternion(quaternion);
}

Rotation3D::Rotation3D(double heading, double attitude, double bank,
		AngleUnit angleUnit) {
	setEuler(heading, attitude, bank, angleUnit);
}

Rotation3D::Rotation3D(const Vector3d &vec, AngleUnit angleUnit) {
	setEuler(vec, angleUnit);
}

Rotation3D::Rotation3D(const Matrix3d &mat) :
	mat(mat) {
}

Rotation3D::~Rotation3D() {
}

void Rotation3D::setMatrix(const Matrix3d &mat) {
	this->mat = mat;
}

void Rotation3D::setAxisAngle(double x, double y, double z, double angle,
		AngleUnit angleUnit) {

	Vector3d axis(x, y, z);
	setAxisAngle(axis, angle, angleUnit);
}

void Rotation3D::setAxisAngle(const Vector3d &axis, double angle,
		AngleUnit angleUnit) {

	double a = UnitConverter::convert(angle, angleUnit, DEGREE);
	Vector3d myAxis = axis;
	myAxis.normalize();

	Quatd q = Quatd::fromAxisRot(myAxis, a);
	setQuaternion(q);
}

void Rotation3D::setQuaternion(const Quatd &quaternion) {
	Quatd q(quaternion);
	q.normalize();
	mat = q.rotMatrix();
}

void Rotation3D::setEuler(double heading, double attitude, double bank,
		AngleUnit angleUnit) {

	double h = UnitConverter::convert(heading, angleUnit, RADIAN);
	double a = UnitConverter::convert(attitude, angleUnit, RADIAN);
	double b = UnitConverter::convert(bank, angleUnit, RADIAN);

	// Algorithm from here:
	// http://www.euclideanspace.com/maths/geometry/rotations/conversions/eulerToMatrix/index.htm

	double ch = cos(h);
	double sh = sin(h);
	double ca = cos(a);
	double sa = sin(a);
	double cb = cos(b);
	double sb = sin(b);

	mat.at(0, 0) = ch * ca;
	mat.at(1, 0) = sh * sb - ch * sa * cb;
	mat.at(2, 0) = ch * sa * sb + sh * cb;
	mat.at(0, 1) = sa;
	mat.at(1, 1) = ca * cb;
	mat.at(2, 1) = -ca * sb;
	mat.at(0, 2) = -sh * ca;
	mat.at(1, 2) = sh * sa * cb + ch * sb;
	mat.at(2, 2) = -sh * sa * sb + ch * cb;
}

void Rotation3D::setEuler(const Vector3d &vec, AngleUnit angleUnit) {

	setEuler(vec.x, vec.y, vec.z, angleUnit);
}

Vector3d Rotation3D::getEuler(AngleUnit angleUnit) const {

	// Algorithm from here:
	// http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToEuler/index.htm

	double heading, attitude, bank;

	if (mat.at(0, 1) > 0.99999) {
		// singularity at north pole
		heading = atan2(mat.at(2, 0), mat.at(2, 2));
		attitude = M_PI_2;
		bank = 0;
	} else if (mat.at(0, 1) < -0.99999) {
		// singularity at south pole
		heading = atan2(mat.at(2, 0), mat.at(2, 2));
		attitude = -M_PI_2;
		bank = 0;
	} else {
		heading = atan2(-mat.at(0, 2), mat.at(0, 0));
		bank = atan2(-mat.at(2, 1), mat.at(1, 1));
		attitude = asin(mat.at(0, 1));
	}

	double h = UnitConverter::convert(heading, RADIAN, angleUnit);
	double a = UnitConverter::convert(attitude, RADIAN, angleUnit);
	double b = UnitConverter::convert(bank, RADIAN, angleUnit);

	return Vector3d(h, a, b);
}

Matrix3d Rotation3D::getMatrix() const {
	return mat;
}

void Rotation3D::getAxisAngle(Vector3d &axis, double &angle,
		AngleUnit angleUnit) const {

	// Algorithm from here:
	// http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToAngle/index.htm

	Quatd q = getQuaternion();
	if (q.w > 1)
		q.normalize();
	angle = 2 * acos(q.w);
	double s = sqrt(1 - q.w * q.w);
	if (s < 0.00001) {
		axis.x = q.v.x;
		axis.y = q.v.y;
		axis.z = q.v.z;
	} else {
		axis.x = q.v.x / s;
		axis.y = q.v.y / s;
		axis.z = q.v.z / s;
	}

	angle = UnitConverter::convert(angle, RADIAN, angleUnit);
}
Quatd Rotation3D::getQuaternion() const {
	return Quatd::fromMatrix(mat);
}

string Rotation3D::getBaseTag() {
	return "ROTATION3D";
}

string Rotation3D::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<MATRIX />" + "</" + getBaseTag() + ">";
}

Location Rotation3D::createValueLocation(double value) const {
	stringstream ss;
	ss << value;
	Location loc("<VALUE />", "VALUE");
	loc = ss.str();
	return loc;
}

void Rotation3D::fillInto(Location &loc) const {

	Type::fillInto(loc);

	Location matrix = loc[XPath("MATRIX")];

	matrix.add(createValueLocation(mat.at(0, 0)));
	matrix.add(createValueLocation(mat.at(0, 1)));
	matrix.add(createValueLocation(mat.at(0, 2)));
	matrix.add(createValueLocation(mat.at(1, 0)));
	matrix.add(createValueLocation(mat.at(1, 1)));
	matrix.add(createValueLocation(mat.at(1, 2)));
	matrix.add(createValueLocation(mat.at(2, 0)));
	matrix.add(createValueLocation(mat.at(2, 1)));
	matrix.add(createValueLocation(mat.at(2, 2)));
}

void Rotation3D::fromLocation(const Location &loc, Rotation3D &object) {

	Type::fromLocation(loc, object);
	Location matrix = loc[XPath("MATRIX")];
	Matrix3d mat;
	try {
		for (int i = 1; i <= 9; i++) {
			stringstream ss;
			ss << "VALUE[" << i << "]";
			mat.data[i - 1] = extract<double> (matrix[XPath(ss.str())]);
		}
	} catch (ParsingException &e) {
		throw ParsingException("wrong number of matrix entries. Required: 9",
				matrix.getDocumentText());
	}

	object.setMatrix(mat);
}

void Rotation3D::sanitizeLocation(Location &loc) const {
	XPath xp(getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

bool Rotation3D::operator==(const Rotation3D &other) const {
	return Type::operator ==(other) && other.mat == this->mat;
}
