/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * MovementDataTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>
#include <cmath>

#include "btl/PersonData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(PersonDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		PersonData original("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setFacing(true);
		original.setHasNeighbour(true);
		original.setPoi(false);
		original.setTalking(true);
		original.setWalking(false);
		original.setAngle(0.23);
		original.setDistance(-12.45);
		original.setGaze(213.56);
		original.setNeighbourDistance(42.123);
		original.setIdentId(42);
		original.setNeighbourId(7);
		original.setIdentName("foobar");

		boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
		face->setGenerator("facesensor");
		face->setAngle(0.234);
		face->setDistance(13.0345);
		face->setHeight(-234.4);
		original.setFace(face);

		boost::shared_ptr<Legs> legs = boost::shared_ptr<Legs>(new Legs);
		legs->setGenerator("legsensor");
		legs->setAngle(-23.45);
		legs->setDistance(2134.34);
		legs->setAngleVariance(0.45);
		legs->setDistanceVariance(1.34);
		legs->setNumInvolved(2134);
		legs->setInvolved1(4543);
		legs->setInvolved2(1223);
		legs->setPair(true);
		original.setLegs(legs);

		Location originalLoc(PersonData::getTemplate(), PersonData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		PersonData parsed = extract<PersonData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.isFacing(), original.isFacing());
		BOOST_REQUIRE_EQUAL(parsed.hasNeighbour(), original.hasNeighbour());
		BOOST_REQUIRE_EQUAL(parsed.isPoi(), original.isPoi());
		BOOST_REQUIRE_EQUAL(parsed.isTalking(), original.isTalking());
		BOOST_REQUIRE_EQUAL(parsed.isWalking(), original.isWalking());
		BOOST_REQUIRE_CLOSE(parsed.getAngle(), original.getAngle(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getDistance(), original.getDistance(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getGaze(), original.getGaze(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getNeighbourDistance(), original.getNeighbourDistance(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getIdentId(), original.getIdentId());
		BOOST_REQUIRE_EQUAL(parsed.getNeighbourId(), original.getNeighbourId());
		BOOST_REQUIRE_EQUAL(parsed.getIdentName(), original.getIdentName());
		BOOST_REQUIRE_EQUAL(*(parsed.getFace()), *(original.getFace()));
		BOOST_REQUIRE_EQUAL(*(parsed.getLegs()), *(original.getLegs()));

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	try {

		Location fileContents(readFile("Person.xml"), PersonData::getBaseTag());
		PersonData parsed = extract<PersonData>(fileContents);

		BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
		BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());

		BOOST_REQUIRE_CLOSE((float) 2.1467, parsed.getAngle(), 0.01);
		BOOST_REQUIRE_CLOSE((float) 1235, parsed.getDistance(), 0.001);
		BOOST_REQUIRE_EQUAL(false, parsed.isFacing());
		BOOST_REQUIRE_CLOSE((float) (M_PI / 2.0), parsed.getGaze(), 0.01);
		BOOST_REQUIRE_EQUAL(123, parsed.getIdentId());
		BOOST_REQUIRE_EQUAL("peter", parsed.getIdentName());
		BOOST_REQUIRE_CLOSE((float) 456, parsed
				.getNeighbourDistance(), 0.0001);
		BOOST_REQUIRE_EQUAL(12, parsed.getNeighbourId());
		BOOST_REQUIRE_EQUAL(true, parsed.isPoi());
		BOOST_REQUIRE_EQUAL(true, parsed.isTalking());
		BOOST_REQUIRE_EQUAL(true, parsed.isWalking());

		Face face;
		face.setTimestamp(Timestamp(12345, 67890));
		face.setGenerator("facesensor");
		face.setAngle(0.45);
		face.setDistance(77.23);
		face.setHeight(-12.34);
		face.setAngleVariance(0.34);
		face.setHeightVariance(12.12);
		face.setDistanceVariance(-3.45);
		face.setLineOfSight(30);
		face.setFaceWidthAngle(13.56);
		face.setReliability(0.67);
		face.setImagePositionX(123);
		face.setImagePositionY(92);
		face.setImageWidth(200);
		face.setImageHeight(42);
		BOOST_REQUIRE(parsed.hasFace());
		BOOST_REQUIRE_EQUAL(face, *(parsed.getFace()));

		Legs legs;
		legs.setTimestamp(Timestamp(12345, 67890));
		legs.setGenerator("legsensor");
		legs.setAngle(0.45);
		legs.setDistance(77.23);
		legs.setAngleVariance(0.45);
		legs.setDistanceVariance(1.34);
		legs.setNumInvolved(2134);
		legs.setInvolved1(4543);
		legs.setInvolved2(1223);
		legs.setPair(true);
		BOOST_REQUIRE(parsed.hasLegs());
		BOOST_REQUIRE_EQUAL(legs, *(parsed.getLegs()));

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testVariablePercepts) {

	try {

		PersonData original("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setFacing(true);
		original.setHasNeighbour(true);
		original.setPoi(false);
		original.setTalking(true);
		original.setWalking(false);
		original.setAngle(0.23);
		original.setDistance(-12.45);
		original.setGaze(213.56);
		original.setNeighbourDistance(42.123);
		original.setIdentId(42);
		original.setNeighbourId(7);
		original.setIdentName("foobar");

		BOOST_REQUIRE(!original.hasFace());
		BOOST_REQUIRE(!original.hasLegs());

		Location originalLoc(PersonData::getTemplate(), PersonData::getBaseTag());
		originalLoc = original;

		cout << "### Person without percepts:" << endl << originalLoc.getDocumentText(true) << endl;

		PersonData parsed = extract<PersonData>(originalLoc);
		BOOST_REQUIRE(!parsed.hasFace());
		BOOST_REQUIRE(!parsed.hasLegs());

		boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
		face->setGenerator("facesensor");
		face->setAngle(0.234);
		face->setDistance(13.0345);
		face->setHeight(-234.4);
		original.setFace(face);

		original.setFace(face);
		BOOST_REQUIRE(original.hasFace());

		Location originalLoc2(PersonData::getTemplate(), PersonData::getBaseTag());
		originalLoc2 = original;

		cout << "### Person with face:" << endl << originalLoc2.getDocumentText(true) << endl;

		parsed = extract<PersonData>(originalLoc2);
		BOOST_REQUIRE(parsed.hasFace());
		BOOST_REQUIRE_EQUAL(*face, *(parsed.getFace()));
		BOOST_REQUIRE(!parsed.hasLegs());

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception (") + typeid(e).name() + "): " + e.what());
	}

}

BOOST_AUTO_TEST_SUITE_END()
