/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data;

import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.common.MicroTimestamp;

/**
 * This class is used to store data from the odometry sensors. It holds the
 * xPosition[m], yPosition[m] and yaw[rad] of the robot in world coordinates. It
 * also contains the time of these sensor values as a timestamp.
 * 
 * @author unknown
 * @author jwienke
 */
public class OdometryData extends PositionData implements Ticked {

	private MicroTimestamp readTime = new MicroTimestamp();

	public OdometryData() {
		super();
	}

	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("#ODOMETRY# ");
		strb.append("timestamp: " + getTimestamp() + "; ");
		strb.append("x: " + getX() + "; ");
		strb.append("y: " + getY() + "; ");
		strb.append("yaw: " + getYaw() + ";");
		return strb.toString();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element readTimeElement = new Element(MicroTimestamp.getBaseTag());
		parent.appendChild(readTimeElement);
		readTime.fillInto(readTimeElement);

	}

	public static String getBaseTag() {
		return "ROBOTPOSITION";
	}

	public static void fromElement(Element element, OdometryData type)
			throws ParsingException {
		PositionData.fromElement(element, type);

		Nodes readTimeNodes = element.query(MicroTimestamp.getBaseTag());
		if (readTimeNodes.size() < 1
				|| !(readTimeNodes.get(0) instanceof Element)) {
			throw new ParsingException("Cannot find element with name "
					+ MicroTimestamp.getBaseTag());
		}

		MicroTimestamp.fromElement((Element) readTimeNodes.get(0),
				type.getReadTime());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MicroTimestamp getReadTime() {
		return readTime;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeMicroSeconds() {
		return readTime.getMicroSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeSeconds() {
		return readTime.getSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setReadTime(MicroTimestamp readTime) {
		this.readTime = readTime;
	}

}
