/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.information;

import java.util.HashMap;
import java.util.Map;

/**
 * This is a helper class that provides parameters and high level information
 * about different camera models. This includes helper methods for handy
 * conversions.
 * 
 * @author lziegler
 */
public class CameraInfo {

	/**
	 * An enum containing the known camera models.
	 */
	public static enum CameraModel {
		SONY_EVI_D31, LOGITECH_PRO_9000, POINTGRAY_GRAS_20S4C_C_0_3M, POINTGRAY_GRAS_20S4C_C_1_4M, POINTGRAY_GRAS_20S4C_C_2_0M
	}

	/**
	 * A {@link HashMap} containing the ranges of each camera model. The values
	 * are ranges as degree values.
	 */
	private static final Map<CameraModel, CameraInfo> info = new HashMap<CameraModel, CameraInfo>() {
		private static final long serialVersionUID = 1257854590263319352L;
		{
			put(CameraModel.SONY_EVI_D31, new CameraInfo(12, // max zoom factor
					4.3, 48.8, // angle of view
					752, 585, // resolution
					-100, 100, // angle pan
					-25, 25)); // angle tilt

			put(CameraModel.LOGITECH_PRO_9000, new CameraInfo(1, // max zoom
																 // factor
					74, 74, // angle of view
					800, 600, // resolution
					0, 0, // angle pan
					0, 0)); // angle tilt
			put(CameraModel.POINTGRAY_GRAS_20S4C_C_0_3M, new CameraInfo(1, // max
																		   // zoom
																		   // factor
					74, 74, // angle of view
					640, 480, // resolution
					0, 0, // angle pan
					0, 0)); // angle tilt
			put(CameraModel.POINTGRAY_GRAS_20S4C_C_1_4M, new CameraInfo(1, // max
																		   // zoom
																		   // factor
					74, 74, // angle of view
					1384, 1036, // resolution
					0, 0, // angle pan
					0, 0)); // angle tilt
			put(CameraModel.POINTGRAY_GRAS_20S4C_C_2_0M, new CameraInfo(1, // max
																		   // zoom
																		   // factor
					74, 74, // angle of view
					640, 480, // resolution
					1624, 1224, // angle pan
					0, 0)); // angle tilt
		}
	};

	/**
	 * Getter for the information about the given camera model.
	 * 
	 * @param model
	 *            The camera model to receive information for.
	 * @return {@link CameraInfo} instance, that describes the camera.
	 */
	public static CameraInfo getInfo(CameraModel model) {
		return info.get(model);
	}

	/**
	 * Calculates the horizontal angle of view for a given zoom position.
	 * 
	 * @param camera
	 *            The used camera model.
	 * @param zoom
	 *            The current zoom position of the camera.
	 * @return The angle of view corresponding to the zoom position of the given
	 *         camera. [radiant value]
	 */
	public static double calculateAngleOfView(CameraModel camera, double zoom) {

		if (info.get(camera).getMaxAngleOfViewHRad() == info.get(camera)
				.getMinAngleOfViewHRad()) {
			return info.get(camera).getMaxAngleOfViewHRad();
		}

		double angle = (1 - zoom)
				* (info.get(camera).getMaxAngleOfViewHRad() - info.get(camera)
						.getMinAngleOfViewHRad())
				+ info.get(camera).getMinAngleOfViewHRad();
		return angle;
	}

	/**
	 * Converts a angle value from degree to radiant representation.
	 * 
	 * @param angle
	 *            The angle as degree value.
	 * @return The angle as radiant value.
	 */
	public static double degreeToRadiant(double angle) {
		return angle * Math.PI / 180.0;
	}

	/**
	 * Converts a angle value from radiant to degree representation.
	 * 
	 * @param angle
	 *            The angle as radiant value.
	 * @return The angle as degree value.
	 */
	public static double radiantToDegree(double angle) {
		return angle * 180.0 / Math.PI;
	}

	private int maxZoomFactor;
	private double minAngleOfViewH;
	private double maxAngleOfViewH;
	private double minAnglePan;
	private double maxAnglePan;
	private double minAngleTilt;
	private double maxAngleTilt;
	private int resolutionX;
	private int resolutionY;

	/**
	 * Creates a CameraInfo instance. Attention: given angle values must be in
	 * degree representation.
	 * 
	 * @param maxZoomFactor
	 *            maximal zoom factor.
	 * @param minAngleOfViewH
	 *            angle of view (h) at maximal zoom position. [degree]
	 * @param maxAngleOfViewH
	 *            angle of view (h) at minimal zoom position. [degree]
	 * @param resolutionX
	 *            resolution in x direction.
	 * @param resolutionY
	 *            resolution in y direction.
	 * @param minAnglePan
	 *            minimal pan angle. This should be the most right position
	 *            possible [degree; 0 = straight; negative = right; positive =
	 *            left]
	 * @param maxAnglePan
	 *            maximal pan angle. This should be the most left position
	 *            possible [degree; 0 = straight; negative = right; positive =
	 *            left]
	 * @param minAngleTilt
	 *            minimal tilt angle. This should be the most downwards position
	 *            possible [degree; 0 = straight; negative = down; positive =
	 *            up]
	 * @param maxAngleTilt
	 *            maximal tilt angle. This should be the most upwards position
	 *            possible [degree; 0 = straight; negative = down; positive =
	 *            up]
	 */
	public CameraInfo(int maxZoomFactor, double minAngleOfViewH,
			double maxAngleOfViewH, int resolutionX, int resolutionY,
			double minAnglePan, double maxAnglePan, double minAngleTilt,
			double maxAngleTilt) {
		this.maxZoomFactor = maxZoomFactor;
		this.minAngleOfViewH = minAngleOfViewH;
		this.maxAngleOfViewH = maxAngleOfViewH;
		this.resolutionX = resolutionX;
		this.resolutionY = resolutionY;
		this.minAnglePan = minAnglePan;
		this.maxAnglePan = maxAnglePan;
		this.minAngleTilt = minAngleTilt;
		this.maxAngleTilt = maxAngleTilt;
	}

	/**
	 * @return Maximal zoom factor. [factor]
	 */
	public int getMaxZoomFactor() {
		return maxZoomFactor;
	}

	/**
	 * Angle of view (h) at maximal zoom position. [degree]
	 * 
	 * @return Angle of view (h) [degree]
	 */
	public double getMinAngleOfViewHDeg() {
		return minAngleOfViewH;
	}

	/**
	 * Angle of view (h) at minimal zoom position. [degree]
	 * 
	 * @return Angle of view (h) [degree]
	 */
	public double getMaxAngleOfViewHDeg() {
		return maxAngleOfViewH;
	}

	/**
	 * Angle of view (h) at maximal zoom position. [radiant]
	 * 
	 * @return Angle of view (h) [radiant]
	 */
	public double getMinAngleOfViewHRad() {
		return degreeToRadiant(minAngleOfViewH);
	}

	/**
	 * Angle of view (h) at minimal zoom position. [radiant]
	 * 
	 * @return Angle of view (h) [radiant]
	 */
	public double getMaxAngleOfViewHRad() {
		return degreeToRadiant(maxAngleOfViewH);
	}

	/**
	 * Resolution of the cameras sensor.
	 * 
	 * @return Resolution in y direction.
	 */
	public int getResolutionY() {
		return resolutionY;
	}

	/**
	 * Resolution of the cameras sensor.
	 * 
	 * @return Resolution in x direction.
	 */
	public int getResolutionX() {
		return resolutionX;
	}

	/**
	 * @return Minimal pan angle. This should be the most right position
	 *         possible [degree; 0 = straight; negative = right; positive =
	 *         left]
	 */
	public double getMinAnglePanDeg() {
		return minAnglePan;
	}

	/**
	 * @return Maximal pan angle. This should be the most left position possible
	 *         [degree; 0 = straight; negative = right; positive = left]
	 */
	public double getMaxAnglePanDeg() {
		return maxAnglePan;
	}

	/**
	 * 
	 * @return Minimal tilt angle. This should be the most downwards position
	 *         possible [degree; 0 = straight; negative = down; positive = up]
	 */
	public double getMinAngleTiltDeg() {
		return minAngleTilt;
	}

	/**
	 * 
	 * @return Maximal tilt angle. This should be the most upwards position
	 *         possible [degree; 0 = straight; negative = down; positive = up]
	 */
	public double getMaxAngleTiltDeg() {
		return maxAngleTilt;
	}

	/**
	 * @return Minimal pan angle. This should be the most right position
	 *         possible [radiant; 0 = straight; negative = right; positive =
	 *         left]
	 */
	public double getMinAnglePanRad() {
		return degreeToRadiant(minAnglePan);
	}

	/**
	 * @return Maximal pan angle. This should be the most left position possible
	 *         [radiant; 0 = straight; negative = right; positive = left]
	 */
	public double getMaxAnglePanRad() {
		return degreeToRadiant(maxAnglePan);
	}

	/**
	 * 
	 * @return Minimal tilt angle. This should be the most downwards position
	 *         possible [radiant; 0 = straight; negative = down; positive = up]
	 */
	public double getMinAngleTiltRad() {
		return degreeToRadiant(minAngleTilt);
	}

	/**
	 * 
	 * @return Maximal tilt angle. This should be the most upwards position
	 *         possible [radiant; 0 = straight; negative = down; positive = up]
	 */
	public double getMaxAngleTiltRad() {
		return degreeToRadiant(maxAngleTilt);
	}
}
